/*
ds18b20 lib 0x02

copyright (c) Davide Gironi, 2012

Released under GPLv3.
Please refer to LICENSE file for licensing information.
*/
 
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <avr/io.h>



#include "param.h"
#include "ds18b20.h"



//----------------------------------------
// Reset DS18S20
//----------------------------------------
uint8_t ds1820_reset(uint8_t used_pin)
{
  uint8_t err=100;
  DS1820_DDR |= 1<<used_pin;						// define as ouput
  DS1820_PORT &= ~(1<<used_pin);					//Pull low
  _delay_us(480);				;					// 480 us
  DS1820_DDR &= ~(1<<used_pin);						// define as input
  DS1820_PORT |= 1<<used_pin;						//Pullup on
  _delay_us(66);										// 66 us
  err = (DS1820_PIN & (1<<used_pin)) >> used_pin;	// no presence detect --> err=1 otherwise err=0
  _delay_us(240);									// 240 us
  if( (DS1820_PIN & (1<<used_pin)) == 0 ){			// short circuit --> err=2
    err = 2;
	}
  return err;
}
//-----------------------------------------
// Write one bit to DS18S20
//-----------------------------------------
void ds1820_wr_bit(uint8_t wrbit,uint8_t used_pin)
{
	if (wrbit ==0)	{
		DS1820_DDR |= 1<<used_pin;					// define as ouput
  		DS1820_PORT &= ~(1<<used_pin);				//Pull low
		_delay_us(60);
		DS1820_DDR &= ~(1<<used_pin);				// define as input
	  	DS1820_PORT |= 1<<used_pin;					//Pullup on
		_delay_us(4);
	}
	if (wrbit ==1)	{
		DS1820_DDR |= 1<<used_pin;					// define as ouput
  		DS1820_PORT &= ~(1<<used_pin);				//Pull low
		_delay_us(10);
		DS1820_DDR &= ~(1<<used_pin);				// define as input
	  	DS1820_PORT |= 1<<used_pin;					//Pullup on
		_delay_us(54);
	}
}
//-----------------------------------------
// Read one bit from DS18S20
//-----------------------------------------
uint8_t ds1820_re_bit(uint8_t used_pin)
{
	uint8_t rebit;
	DS1820_DDR |= 1<<used_pin;							// define as ouput
  	DS1820_PORT &= ~(1<<used_pin);						//Pull low
	_delay_us(1);
	DS1820_DDR &= ~(1<<used_pin);						// define as input
	DS1820_PORT |= 1<<used_pin;							//Pullup on
	_delay_us(10);
	rebit = (DS1820_PIN & (1<<used_pin)) >> used_pin; 	//Read bit
	_delay_us(50);
	return rebit;								
}
//-----------------------------------------
// Read 1 byte from DS18S20
//-----------------------------------------
uint8_t ds1820_re_byte(uint8_t used_pin)
{
	uint8_t rebyte =0x00;
	uint8_t rebit;
	uint8_t i;

	for (i=0;i<8;i++)
	{
		rebit=ds1820_re_bit(used_pin);
		//delay_us(2);									//be on the save side
		if (rebit==1){
			rebyte|=(1<<i);
		}
	}
	return(rebyte);
}
//-----------------------------------------
// Write 1 byte to DS18S20
//-----------------------------------------
void ds1820_wr_byte(uint8_t wrbyte,uint8_t used_pin)
{
	uint8_t i;
	for (i=0; i<8; i++) // writes byte, one bit at a time
	{	    
		ds1820_wr_bit((wrbyte & 0b00000001),used_pin);
		wrbyte = wrbyte >> 1;
	}
	_delay_us(5);
}
//-----------------------------------------
// Read temperature
//-----------------------------------------
void  ds1820_read_temp(uint8_t used_pin)	
{
    int temp;
    int16_t Vorkommastelle;
    uint8_t Nachkommastelle;	
	
	
	uint8_t error,i;
	uint16_t j=0;
    uint8_t scratchpad[9];
	
	
	
	
	scratchpad[0]=0;
	scratchpad[1]=0;
	scratchpad[2]=0;
	scratchpad[3]=0;
	scratchpad[4]=0;
	scratchpad[5]=0;
	scratchpad[6]=0;
	scratchpad[7]=0;
	scratchpad[8]=0;
	
	cli();
	
	error=ds1820_reset(used_pin);									//1. Reset
	if (error==0){
	    ds1820_wr_byte(0xCC,used_pin);  							//2. skip ROM
	    ds1820_wr_byte(0x44,used_pin);  							//3. ask for temperature conversion
	    while (ds1820_re_byte(used_pin)==0xFF && j<1000){			//4. wait until conversion is finished 
			_delay_us(1);
			j++;
		}									 
	    error=ds1820_reset(used_pin);								//5. Reset
	    ds1820_wr_byte(0xCC,used_pin);  							//6. skip ROM
	    ds1820_wr_byte(0xBE,used_pin);  							//7. Read entire scratchpad 9 bytes
    
	    for (i=0; i<9; i++)     									//8. Get scratchpad byte by byte
	    {
	       scratchpad[i]=ds1820_re_byte(used_pin); 					//9. read one DS18S20 byte
	    }
	}

   sei();
    if( id[1] ) {                // Negativer Temperaturwert
    id[0] ^= 0xFF;               // Einer-Komplement
    id[0]++;                     // inkrementieren
    }
    id[0] >>= 1;                 // rechtes Bit fr 0,5 C rausschieben (durch 2 teilen)
    
  // Temperatur mit 0,1 C Genauigkeit berechnen
    temp = (int)id[0]*100 - 25 + (int)(id[7] - id[6]) * 100 / id[7];
    
     if( id[1])                   // Negativer Temperaturwert
      {
       temp *= -1;
       //temp /= 10; // Zweite Nachkommastelle abtrennen
	  
 	  
	  
	  }

     else
      {
          
      }
 
   

	    Vorkommastelle = temp / 100;
        Nachkommastelle = (temp % 100);      
        Nachkommastelle = Nachkommastelle % 10;// Zweite Nachkommastelle abtrennen
	
        sprintf(msg,"%-03d,%1d", Vorkommastelle, Nachkommastelle);

	
}
//-----------------------------------------
// Initialize DS18S20
//-----------------------------------------
void  ds1820_init(uint8_t used_pin)	
{
	uint8_t error;
	uint16_t i =0;
	error=ds1820_reset(used_pin);									//1. Reset
	if (error==0){
	    ds1820_wr_byte(0xCC,used_pin);  							//2. skip ROM
	    ds1820_wr_byte(0x44,used_pin);  							//3. ask for temperature conversion
	    while (ds1820_re_byte(used_pin)==0xFF && i<1000){			//4. wait until conversion is finished 
			_delay_us(1);
			i++;
		}
		error=ds1820_reset(used_pin);								//5. Reset
	    ds1820_wr_byte(0xCC,used_pin);  							//6. skip ROM
	    ds1820_wr_byte(0xBE,used_pin);  							//7. Read entire scratchpad 9 bytes
		}
    
	   
}









void delay_ms(uint16_t time){
	uint16_t i;
	for (i=1;i<=time;i++){
		_delay_ms(1);
	}
}
void delay_us(uint16_t time){
	uint16_t i;
	for (i=1;i<=time;i++){
		_delay_us(1);
	}
}












